using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.Text;
using System.ComponentModel;
using System.Threading;

namespace C06PKF
{
    class Program
    {

        /* NAG routine declarations copied from flcsdnet64.cs */

        [StructLayout(LayoutKind.Sequential)]/* Force sequential mapping */
        public struct Complex
        {
            public double Real_Part;
            public double Imag_Part;
        }

        [DllImport("NLW6I27DE_nag.dll")]
          public static extern void C06PKF(
                                           ref int JOB,
                                           [In,Out] Complex[] X,
                                           [In,Out] Complex[] Y,
                                           ref int N,
                                           [Out] Complex[] WORK,
                                           ref int IFAIL
                                           );

        static void Main(string[] args)
        {
            HiPerfTimer pt = new HiPerfTimer();
            int ifail;
            int n = 5000000;
            Complex[] x = new Complex[n];
            Complex[] y = new Complex[n];
            Complex[] work = new Complex[4 * n];

            for (int j=0; j<n; j++)
            {
                  x[j].Real_Part=(double)1.0+Math.Cos(j);
                  x[j].Imag_Part=(double)-0.5+Math.Sin(j);
            }


            for (int i = 0; i < n; i++)
            {
                  y[i].Real_Part = (double)0.5 + Math.Sin(i);
                  y[i].Imag_Part = (double)-0.25+Math.Cos(i);
            }

            pt.Start();

            ifail = 1;
            int job = 2;

            C06PKF(ref job, x, y, ref n, work, ref ifail);

            pt.Stop();
            Console.WriteLine("Duration: {0} sec\n", pt.Duration);
        }
    }

    internal class HiPerfTimer
    {
        [DllImport("Kernel32.dll")]
        private static extern bool QueryPerformanceCounter(
            out long lpPerformanceCount);

        [DllImport("Kernel32.dll")]
        private static extern bool QueryPerformanceFrequency(
            out long lpFrequency);

        private long startTime, stopTime;
        private long freq;

        // Constructor

        public HiPerfTimer()
        {
            startTime = 0;
            stopTime = 0;

            if (QueryPerformanceFrequency(out freq) == false)
            {
                // high-performance counter not supported

                throw new Win32Exception();
            }
        }


        // Start the timer

        public void Start()
        {

            Thread.Sleep(0);

            QueryPerformanceCounter(out startTime);
        }


        // Stop the timer

        public void Stop()
        {
            QueryPerformanceCounter(out stopTime);
        }


        // Returns the duration of the timer (in seconds)

        public double Duration
        {
            get
            {
                return (double)(stopTime - startTime) / (double)freq;
            }
        }
    }
}
